/* real time linux task for reading samples from the SoundCard         */

#define PORT SB

#include <linux/errno.h>
#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/version.h>
#include <asm/io.h> 
#include <rtl.h>
#include <rtl_sched.h>
#include <rtl_fifo.h>
#include <time.h>
#include <pthread.h>
#include "control.h"

pthread_t mytask;
int     sr;
char    channels;

/* read channels from Sound-card */
int kanal0(int port) {
  int  vout=0;
  int  i;

  /* one-byte 8-bit A/D */
  i=inb(port+0x0C);
  while ((i & 128)==128) i=inb(port+0x0C);
  outb(0x20, port+0x0C);
  i=inb(port+0x0E);
  while ((i & 128)==0) i=inb(port+0x0E);
  vout=inb(port+0x0A);
  return(vout*16);
}

/* .................................. */
/* function to write samples to fifos */
/* .................................. */
void *fun(void *t) {
  int   data, rc=0;

  int kanal0(int port);

  while(1) {
    /* FIFO 0 transfers the samples from channel 1                              */
    /* FIFO 9 is an indicator whether a FIFO is full when trying to write to it */

    if (channels==0) pthread_wait_np();
    if ((channels & 1)==1) {
      data=kanal0(PORT);
      if (rtf_put(0, (char *) &data, 2) < 0)
	rtf_put(9, (char *) &rc, sizeof(int));
      pthread_wait_np();
    }
  }
}

int my_handler(unsigned int fifo) {
  struct my_msg_struct msg;
  int err;

  /* FIFO 8 transfers commands to control the driver */
  while ((err=rtf_get(8, &msg, sizeof(msg))) == sizeof(msg)) {
    switch(msg.command) {
    case START_TASK:
      pthread_make_periodic_np(mytask, gethrtime(), HRTICKS_PER_SEC*(float)(1.0/(float)sr));
      pthread_wakeup_np(mytask);
      break;
    case STOP_TASK:
      pthread_suspend_np(mytask);
      break;
    case SET_CH:
      channels=(unsigned char)msg.value;
      break;
      /*
	case SET_POL:
	if ((msg.value & 1)==1) bipol[0]=1; else bipol[0]=0;
	if ((msg.value & 2)==2) bipol[1]=1; else bipol[1]=0;
	break;
	*/
    case SET_SR:
      sr=msg.value;
      break;
    default:
      return -EINVAL;
    }
  }
  if (err!=0) {
    return -EINVAL;
  }
  return 0;
}

int init_module(void) {
  pthread_attr_t     attr;
  struct sched_param sched_param;
  int                i;
  hrtime_t           now;

  /* DSP reset */
  outb(1, PORT+0x06);
  now=gethrtime();
  while(gethrtime()-now < (NSECS_PER_SEC * 0.1)) {;};
  outb(0, PORT+0x06);
  i=inb(PORT+0x0E);
  while((i & 128)==0) {
    i=inb(PORT+0x0E);
  }
  i=inb(PORT+0x0A);
  while (i!=0xAA) i=inb(PORT+0x0A);
  /* set parameters */
  sr=1;
  channels=0x00;
  rtf_create(0,10000);
  rtf_create(8,100);
  rtf_create(9,100);

  pthread_attr_init(&attr);
  sched_param.sched_priority = 2;
  pthread_attr_setschedparam(&attr, &sched_param);
  pthread_create(&mytask,  &attr, fun, (void *)1);
  pthread_setfp_np(mytask,1);
  rtf_create_handler(8, &my_handler);
  return 0;
}


void cleanup_module(void)
{
  rtf_destroy(0);
  rtf_destroy(8);
  rtf_destroy(9);
  pthread_delete_np(mytask);
}



